@@ -1,23 +1,12 @@
-import os
-
-from PIL import Image, ExifTags
+"""Base datastructures for manipulated images."""
+from PIL import Image
 
 from django.core.files.uploadedfile import InMemoryUploadedFile
 
-from ..settings import (
-    USE_PLACEHOLDIT,
-    VERSATILEIMAGEFIELD_PLACEHOLDER_IMAGE
-)
-from ..utils import get_image_metadata_from_file_ext
+from ..settings import QUAL, VERSATILEIMAGEFIELD_PROGRESSIVE_JPEG
+from ..utils import get_image_metadata_from_file
 
-if not USE_PLACEHOLDIT:
-    PLACEHOLDER_FOLDER, PLACEHOLDER_FILENAME = os.path.split(
-        VERSATILEIMAGEFIELD_PLACEHOLDER_IMAGE
-    )
-
-ORIENTATION_KEY = ExifTags.TAGS.keys()[
-    ExifTags.TAGS.values().index('Orientation')
-]
+EXIF_ORIENTATION_KEY = 274
 
 
 class ProcessedImage(object):
@@ -39,18 +28,26 @@ class ProcessedImage(object):
     the `preprocess` method for more specific information.
     """
 
-    def __init__(self, path_to_image, storage, create_on_demand):
+    name = None
+    url = None
+
+    def __init__(self, path_to_image, storage, create_on_demand,
+                 placeholder_image=None):
+        """Construct a ProcessedImage."""
         self.path_to_image = path_to_image
         self.storage = storage
         self.create_on_demand = create_on_demand
+        self.placeholder_image = placeholder_image
 
     def process_image(self, image, image_format, **kwargs):
         """
+        Ensure NotImplemented is raised if not overloaded by subclasses.
+
         Arguments:
             * `image`: a PIL Image instance
             * `image_format`: str, a valid PIL format (i.e. 'JPEG' or 'GIF')
 
-        Returns a StringIO.StringIO representation of the resized image.
+        Returns a BytesIO representation of the resized image.
 
         Subclasses MUST implement this method.
         """
@@ -60,7 +57,7 @@ class ProcessedImage(object):
 
     def preprocess(self, image, image_format):
         """
-        Preprocesses an image.
+        Preprocess an image.
 
         An API hook for image pre-processing. Calls any image format specific
         pre-processors (if defined). I.E. If `image_format` is 'JPEG', this
@@ -84,17 +81,16 @@ class ProcessedImage(object):
             exif_datadict = image._getexif()  # returns None if no EXIF data
             if exif_datadict is not None:
                 exif = dict(exif_datadict.items())
-                try:
-                    orientation = exif[ORIENTATION_KEY]
-                except KeyError:
-                    pass
-                else:
-                    if orientation == 3:
-                        image = image.transpose(Image.ROTATE_180)
-                    elif orientation == 6:
-                        image = image.transpose(Image.ROTATE_270)
-                    elif orientation == 8:
-                        image = image.transpose(Image.ROTATE_90)
+                orientation = exif.get(EXIF_ORIENTATION_KEY, None)
+                if orientation == 3:
+                    image = image.transpose(Image.ROTATE_180)
+                elif orientation == 6:
+                    image = image.transpose(Image.ROTATE_270)
+                elif orientation == 8:
+                    image = image.transpose(Image.ROTATE_90)
+
+        # Ensure any embedded ICC profile is preserved
+        save_kwargs['icc_profile'] = image.info.get('icc_profile')
 
         if hasattr(self, 'preprocess_%s' % image_format):
             image, addl_save_kwargs = getattr(
@@ -105,20 +101,43 @@ class ProcessedImage(object):
 
         return image, save_kwargs
 
-    def retrieve_image(self, path_to_image):
+    def preprocess_GIF(self, image, **kwargs):
         """
-        Returns a PIL Image instance stored at `path_to_image`
+        Receive a PIL Image instance of a GIF and return 2-tuple.
 
-        If `path_to_image` is None, the global Placeholder image (as specified
-        by the VERSATILEIMAGEFIELD_PLACEHOLDER_IMAGE setting) will be returned.
+        Args:
+            * [0]: Original Image instance (passed to `image`)
+            * [1]: Dict with a transparency key (to GIF transparency layer)
         """
-        if not path_to_image:
-            image = VERSATILEIMAGEFIELD_PLACEHOLDER_IMAGE
-            file_ext = image.rsplit('.')[-1]
+        if 'transparency' in image.info:
+            save_kwargs = {'transparency': image.info['transparency']}
         else:
-            image = self.storage.open(path_to_image, 'r')
-            file_ext = path_to_image.rsplit('.')[-1]
-        image_format, mime_type = get_image_metadata_from_file_ext(file_ext)
+            save_kwargs = {}
+        return (image, save_kwargs)
+
+    def preprocess_JPEG(self, image, **kwargs):
+        """
+        Receive a PIL Image instance of a JPEG and returns 2-tuple.
+
+        Args:
+            * [0]: Image instance, converted to RGB
+            * [1]: Dict with a quality key (mapped to the value of `QUAL` as
+                   defined by the `VERSATILEIMAGEFIELD_JPEG_RESIZE_QUALITY`
+                   setting)
+        """
+        save_kwargs = {
+            'progressive': VERSATILEIMAGEFIELD_PROGRESSIVE_JPEG,
+            'quality': QUAL
+        }
+        if image.mode != 'RGB':
+            image = image.convert('RGB')
+        return (image, save_kwargs)
+
+    def retrieve_image(self, path_to_image):
+        """Return a PIL Image instance stored at `path_to_image`."""
+        image = self.storage.open(path_to_image, 'rb')
+        image_format, mime_type = get_image_metadata_from_file(image)
+        file_ext = path_to_image.rsplit('.')[-1]
 
         return (
             Image.open(image),
@@ -129,23 +148,22 @@ class ProcessedImage(object):
 
     def save_image(self, imagefile, save_path, file_ext, mime_type):
         """
-        Saves an image to self.storage at `save_path`.
+        Save an image to self.storage at `save_path`.
 
         Arguments:
-            `imagefile`: Raw image data, typically a StringIO instance.
+            `imagefile`: Raw image data, typically a BytesIO instance.
             `save_path`: The path within self.storage where the image should
                          be saved.
             `file_ext`: The file extension of the image-to-be-saved.
             `mime_type`: A valid image mime type (as found in
                          versatileimagefield.utils)
         """
-
         file_to_save = InMemoryUploadedFile(
             imagefile,
             None,
             'foo.%s' % file_ext,
             mime_type,
-            imagefile.len,
+            imagefile.tell(),
             None
         )
         file_to_save.seek(0)
