@@ -1,18 +1,20 @@
+from django.conf import settings
+
 from ..settings import (
-    USE_PLACEHOLDIT,
     cache,
     VERSATILEIMAGEFIELD_CACHE_LENGTH
 )
 from ..utils import get_filtered_path
 
 from .base import ProcessedImage
+from .mixins import DeleteAndClearCacheMixIn
 
 
 class InvalidFilter(Exception):
     pass
 
 
-class FilteredImage(ProcessedImage):
+class FilteredImage(DeleteAndClearCacheMixIn, ProcessedImage):
     """
     A ProcessedImage subclass that applies a filter to an image.
 
@@ -24,21 +26,21 @@ class FilteredImage(ProcessedImage):
                           filename to identify it. This should be short
                           and descriptive (i.e. 'grayscale' or 'invert')
 
-    Subclasses must implement a process_filter method.
+    Subclasses must implement a process_image method.
     """
-    name = None
-    url = None
 
     def __init__(self, path_to_image, storage, create_on_demand, filename_key):
         super(FilteredImage, self).__init__(
             path_to_image, storage, create_on_demand
         )
-        self.name, self.url = get_filtered_path(
+        self.name = get_filtered_path(
             path_to_image=self.path_to_image,
             filename_key=filename_key,
             storage=storage
         )
 
+        self.url = storage.url(self.name)
+
     def create_filtered_image(self, path_to_image, save_path_on_storage):
         """
         Creates a filtered image.
@@ -55,12 +57,14 @@ class FilteredImage(ProcessedImage):
         imagefile = self.process_image(image, image_format, save_kwargs)
         self.save_image(imagefile, save_path_on_storage, file_ext, mime_type)
 
+    def __str__(self):
+        return self.url
+
 
 class DummyFilter(object):
     """
     A 'dummy' version of FilteredImage which is only used if
-    .settings.USE_PLACEHOLDIT is True (i.e. if the
-    VERSATILEIMAGEFIELD_PLACEHOLDER_IMAGE is unset)
+    settings.VERSATILEIMAGEFIELD_USE_PLACEHOLDIT is True
     """
     name = ''
     url = ''
@@ -108,19 +112,24 @@ class FilterLibrary(dict):
                 raise InvalidFilter('`%s` is an invalid filter.' % key)
             else:
                 # Handling 'empty' fields.
-                if not self.original_file_location and USE_PLACEHOLDIT:
-                    # If USE_PLACEHOLDIT is True (i.e.
+                if not self.original_file_location and getattr(
+                    settings, 'VERSATILEIMAGEFIELD_USE_PLACEHOLDIT', False
+                ):
+                    # If VERSATILEIMAGEFIELD_USE_PLACEHOLDIT is True (i.e.
                     # settings.VERSATILEIMAGEFIELD_PLACEHOLDER_IMAGE is unset)
                     # use DummyFilter (so sized renditions can still return
                     # valid http://placehold.it URLs).
                     filtered_path = None
                     prepped_filter = DummyFilter()
                 else:
-                    filtered_path, filtered_url = get_filtered_path(
+                    filtered_path = get_filtered_path(
                         path_to_image=self.original_file_location,
                         filename_key=key,
                         storage=self.storage
                     )
+
+                    filtered_url = self.storage.url(filtered_path)
+
                     filter_cls = self.registry._filter_registry[key]
                     prepped_filter = filter_cls(
                         path_to_image=self.original_file_location,
@@ -154,7 +163,7 @@ class FilterLibrary(dict):
                 # the prepped_filter instance
                 for (
                         attr_name, sizedimage_cls
-                ) in self.registry._sizedimage_registry.iteritems():
+                ) in self.registry._sizedimage_registry.items():
                     setattr(
                         prepped_filter,
                         attr_name,
