@@ -1,37 +1,38 @@
+"""Datastructures for sizing images."""
+from django.conf import settings
 from ..settings import (
-    QUAL,
-    USE_PLACEHOLDIT,
     cache,
     VERSATILEIMAGEFIELD_CACHE_LENGTH
 )
 from ..utils import get_resized_path
 from .base import ProcessedImage
+from .mixins import DeleteAndClearCacheMixIn
 
 
 class MalformedSizedImageKey(Exception):
+    """An Exception for improperly constructured sized image keys."""
+
     pass
 
 
-class SizedImageInstance(object):
-    """
-    A simple class for returning paths-on-storage and URLs
-    associated with images created by SizedImage
-    """
+class SizedImageInstance(DeleteAndClearCacheMixIn):
+    """A simple class for images created by SizedImage."""
 
-    def __init__(self, name, url):
+    def __init__(self, name, url, storage):
+        """Construct a SizedImageInstance."""
         self.name = name
         self.url = url
+        self.storage = storage
 
     def __str__(self):
+        """Return the string representation."""
         return self.url
 
-    def __unicode__(self):
-        return self.__str__()
-
 
 class SizedImage(ProcessedImage, dict):
     """
     A dict subclass that exposes an image sizing API via key access.
+
     Subclasses must implement a `process_image` method.
 
     See versatileimagefield.versatileimagefield.CroppedImage and
@@ -40,6 +41,7 @@ class SizedImage(ProcessedImage, dict):
     """
 
     def __init__(self, path_to_image, storage, create_on_demand, ppoi=None):
+        """Construct a SizedImage."""
         super(SizedImage, self).__init__(
             path_to_image, storage, create_on_demand
         )
@@ -48,20 +50,42 @@ class SizedImage(ProcessedImage, dict):
             key = self.get_filename_key()
         except AttributeError:
             raise NotImplementedError(
-                'SizedImage subclasses must define a'
-                ' `filename_key` attribute or override the '
+                'SizedImage subclasses must define a '
+                '`filename_key` attribute or override the '
                 '`get_filename_key` method.'
             )
         else:
             del key
 
+    def ppoi_as_str(self):
+        """Return PPOI value as a string."""
+        return "%s__%s" % (
+            str(self.ppoi[0]).replace('.', '-'),
+            str(self.ppoi[1]).replace('.', '-')
+        )
+
     def get_filename_key(self):
-        """
-        Returns a string that will be used to identify the resized image.
-        """
+        """Return a string used to identify the resized image."""
         return self.filename_key
 
+    @classmethod
+    def get_filename_key_regex(cls):
+        """Return the filename key regex."""
+        try:
+            return cls.filename_key_regex
+        except AttributeError:
+            try:
+                return cls.filename_key
+            except AttributeError:   # pragma: no cover
+                raise NotImplementedError(
+                    'SizedImage subclasses must define a '
+                    '`filename_key_regex` attribute or a '
+                    '`filename_key` attribute or override the '
+                    '`get_filename_key_regex` class method.'
+                )
+
     def __setitem__(self, key, value):
+        """Ensure attribute assignment is disabled."""
         raise NotImplementedError(
             '%s instances do not allow key'
             ' assignment.' % self.__class__.__name__
@@ -69,7 +93,7 @@ class SizedImage(ProcessedImage, dict):
 
     def __getitem__(self, key):
         """
-        Returns a URL to an image sized according to key.
+        Return a URL to an image sized according to key.
 
         Arguments:
             * `key`: A string in the following format
@@ -78,45 +102,64 @@ class SizedImage(ProcessedImage, dict):
         """
         try:
             width, height = [int(i) for i in key.split('x')]
-        except KeyError:
+        except (KeyError, ValueError):
             raise MalformedSizedImageKey(
                 "%s keys must be in the following format: "
                 "'`width`x`height`' where both `width` and `height` are "
                 "integers." % self.__class__.__name__
             )
 
-        if not self.path_to_image and USE_PLACEHOLDIT:
+        if not self.path_to_image and getattr(
+            settings, 'VERSATILEIMAGEFIELD_USE_PLACEHOLDIT', False
+        ):
             resized_url = "http://placehold.it/%dx%d" % (width, height)
             resized_storage_path = resized_url
         else:
-            resized_storage_path, resized_url = get_resized_path(
+            resized_storage_path = get_resized_path(
                 path_to_image=self.path_to_image,
                 width=width,
                 height=height,
                 filename_key=self.get_filename_key(),
                 storage=self.storage
             )
+
+            try:
+                resized_url = self.storage.url(resized_storage_path)
+            except Exception:  # pragma: no cover
+                resized_url = None
+
             if self.create_on_demand is True:
-                if cache.get(resized_url):
+                if cache.get(resized_url) and resized_url is not None:
                     # The sized path exists in the cache so the image already
                     # exists. So we `pass` to skip directly to the return
                     # statement
                     pass
                 else:
-                    if not self.storage.exists(resized_storage_path):
+                    if resized_storage_path and not self.storage.exists(
+                        resized_storage_path
+                    ):
                         self.create_resized_image(
                             path_to_image=self.path_to_image,
                             save_path_on_storage=resized_storage_path,
                             width=width,
                             height=height
                         )
+
+                        resized_url = self.storage.url(resized_storage_path)
+
                     # Setting a super-long cache for a resized image (30 Days)
                     cache.set(resized_url, 1, VERSATILEIMAGEFIELD_CACHE_LENGTH)
-        return SizedImageInstance(resized_storage_path, resized_url)
+        return SizedImageInstance(
+            name=resized_storage_path,
+            url=resized_url,
+            storage=self.storage
+        )
 
     def process_image(self, image, image_format, save_kwargs,
                       width, height):
         """
+        Process a SizedImage.
+
         Arguments:
             * `image`: a PIL Image instance
             * `image_format`: A valid image mime type (e.g. 'image/jpeg')
@@ -127,37 +170,19 @@ class SizedImage(ProcessedImage, dict):
                         height
 
 
-        Returns a StringIO.StringIO representation of the resized image.
+        Returns a BytesIO representation of the resized image.
 
         Subclasses MUST implement this method.
         """
         raise NotImplementedError(
-            'Subclasses MUST provide a `process_image` method.')
-
-    def preprocess_GIF(self, image, **kwargs):
-        """
-        Receives a PIL Image instance of a GIF and returns 2-tuple:
-            * [0]: Original Image instance (passed to `image`)
-            * [1]: Dict with a transparency key (to GIF transparency layer)
-        """
-        return (image, {'transparency': image.info['transparency']})
-
-    def preprocess_JPEG(self, image, **kwargs):
-        """
-        Receives a PIL Image instance of a JPEG and returns 2-tuple:
-            * [0]: Image instance, converted to RGB
-            * [1]: Dict with a quality key (mapped to the value of `QUAL` as
-                   defined by the `VERSATILEIMAGEFIELD_JPEG_RESIZE_QUALITY`
-                   setting)
-        """
-        if image.mode != 'RGB':
-            image = image.convert('RGB')
-        return (image, {'quality': QUAL})
+            'Subclasses MUST provide a `process_image` method.'
+        )
 
     def create_resized_image(self, path_to_image, save_path_on_storage,
                              width, height):
         """
-        Creates a resized image.
+        Create a resized image.
+
         `path_to_image`: The path to the image with the media directory to
                          resize. If `None`, the
                          VERSATILEIMAGEFIELD_PLACEHOLDER_IMAGE will be used.
