@@ -1,65 +1,41 @@
+from functools import reduce
+
 import os
 
+import magic
 from django.core.exceptions import ImproperlyConfigured
 
 from .settings import (
-    USE_PLACEHOLDIT,
-    VERSATILEIMAGEFIELD_PLACEHOLDER_IMAGE,
+    IMAGE_SETS,
+    QUAL,
+    VERSATILEIMAGEFIELD_POST_PROCESSOR,
     VERSATILEIMAGEFIELD_SIZED_DIRNAME,
-    VERSATILEIMAGEFIELD_FILTERED_DIRNAME,
-    IMAGE_SETS
+    VERSATILEIMAGEFIELD_FILTERED_DIRNAME
 )
 
-if not USE_PLACEHOLDIT:
-    PLACEHOLDER_FOLDER, PLACEHOLDER_FILENAME = os.path.split(
-        VERSATILEIMAGEFIELD_PLACEHOLDER_IMAGE
-    )
-
 # PIL-supported file formats as found here:
 # https://infohost.nmt.edu/tcc/help/pubs/pil/formats.html
-# (PIL Identifier, mime type)
-BMP = ('BMP', 'image/bmp')
-DCX = ('DCX', 'image/dcx')
-EPS = ('eps', 'image/eps')
-GIF = ('GIF', 'image/gif')
-JPEG = ('JPEG', 'image/jpeg')
-PCD = ('PCD', 'image/pcd')
-PCX = ('PCX', 'image/pcx')
-PDF = ('PDF', 'application/pdf')
-PNG = ('PNG', 'image/png')
-PPM = ('PPM', 'image/x-ppm')
-PSD = ('PSD', 'image/psd')
-TIFF = ('TIFF', 'image/tiff')
-XBM = ('XBM', 'image/x-xbitmap')
-XPM = ('XPM', 'image/x-xpm')
-
-# Mapping file extensions to PIL types/mime types
-FILE_EXTENSION_MAP = {
-    'png': PNG,
-    'jpe': JPEG,
-    'jpeg': JPEG,
-    'jpg': JPEG,
-    'gif': GIF,
-    'bmp': BMP,
-    'dib': BMP,
-    'dcx': DCX,
-    'eps': EPS,
-    'ps': EPS,
-    'pcd': PCD,
-    'pcx': PCX,
-    'pdf': PDF,
-    'pbm': PPM,
-    'pgm': PPM,
-    'ppm': PPM,
-    'psd': PSD,
-    'tif': TIFF,
-    'tiff': TIFF,
-    'xbm': XBM,
-    'xpm': XPM
+# {mime type: PIL Identifier}
+MIME_TYPE_TO_PIL_IDENTIFIER = {
+    'image/bmp': 'BMP',
+    'image/dcx': 'DCX',
+    'image/eps': 'eps',
+    'image/gif': 'GIF',
+    'image/jpeg': 'JPEG',
+    'image/pcd': 'PCD',
+    'image/pcx': 'PCX',
+    'application/pdf': 'PDF',
+    'image/png': 'PNG',
+    'image/x-ppm': 'PPM',
+    'image/psd': 'PSD',
+    'image/tiff': 'TIFF',
+    'image/x-xbitmap': 'XBM',
+    'image/x-xpm': 'XPM',
+    'image/webp': 'WEBP',
 }
 
 
-class InvalidSizeList(Exception):
+class InvalidSizeKeySet(Exception):
     pass
 
 
@@ -67,9 +43,17 @@ class InvalidSizeKey(Exception):
     pass
 
 
+def post_process_image_key(image_key):
+    """Apply the processor function associated with settings.VER"""
+    if VERSATILEIMAGEFIELD_POST_PROCESSOR is None:
+        return image_key
+    else:
+        return VERSATILEIMAGEFIELD_POST_PROCESSOR(image_key)
+
+
 def get_resized_filename(filename, width, height, filename_key):
     """
-    Returns the 'resized filename' (according to `width`, `height` and
+    Return the 'resized filename' (according to `width`, `height` and
     `filename_key`) in the following format:
     `filename`-`filename_key`-`width`x`height`.ext
     """
@@ -78,11 +62,21 @@ def get_resized_filename(filename, width, height, filename_key):
     except ValueError:
         image_name = filename
         ext = 'jpg'
-    return "%(image_name)s-%(filename_key)s-%(width)dx%(height)d.%(ext)s" % ({
-        'image_name': image_name,
+
+    resized_template = "%(filename_key)s-%(width)dx%(height)d"
+    if ext.lower() in ['jpg', 'jpeg']:
+        resized_template = resized_template + "-%(quality)d"
+
+    resized_key = resized_template % ({
         'filename_key': filename_key,
         'width': width,
         'height': height,
+        'quality': QUAL
+    })
+
+    return "%(image_name)s-%(image_key)s.%(ext)s" % ({
+        'image_name': image_name,
+        'image_key': post_process_image_key(resized_key),
         'ext': ext
     })
 
@@ -90,15 +84,10 @@ def get_resized_filename(filename, width, height, filename_key):
 def get_resized_path(path_to_image, width, height,
                      filename_key, storage):
     """
-    Returns a 2-tuple to `path_to_image` location on `storage` (position 0)
-    and it's web-accessible URL (position 1) as dictated by `width`, `height`
+    Return a `path_to_image` location on `storage` as dictated by `width`, `height`
     and `filename_key`
     """
-    if not path_to_image:
-        filename = PLACEHOLDER_FILENAME
-        containing_folder = 'GLOBAL-PLACEHOLDER'
-    else:
-        containing_folder, filename = os.path.split(path_to_image)
+    containing_folder, filename = os.path.split(path_to_image)
 
     resized_filename = get_resized_filename(
         filename,
@@ -113,15 +102,12 @@ def get_resized_path(path_to_image, width, height,
         resized_filename
     ]).replace(' ', '')  # Removing spaces so this path is memcached friendly
 
-    return (
-        joined_path,
-        storage.url(joined_path)
-    )
+    return joined_path
 
 
 def get_filtered_filename(filename, filename_key):
     """
-    Returns the 'filtered filename' (according to `filename_key`)
+    Return the 'filtered filename' (according to `filename_key`)
     in the following format:
     `filename`__`filename_key`__.ext
     """
@@ -139,13 +125,9 @@ def get_filtered_filename(filename, filename_key):
 
 def get_filtered_path(path_to_image, filename_key, storage):
     """
-    Returns the 'filtered path' & URL of `path_to_image`
+    Return the 'filtered path'
     """
-    if not path_to_image:
-        filename = PLACEHOLDER_FILENAME
-        containing_folder = 'GLOBAL-PLACEHOLDER'
-    else:
-        containing_folder, filename = os.path.split(path_to_image)
+    containing_folder, filename = os.path.split(path_to_image)
 
     filtered_filename = get_filtered_filename(filename, filename_key)
     path_to_return = os.path.join(*[
@@ -155,27 +137,26 @@ def get_filtered_path(path_to_image, filename_key, storage):
     ])
     # Removing spaces so this path is memcached key friendly
     path_to_return = path_to_return.replace(' ', '')
-    return (
-        path_to_return,
-        storage.url(path_to_return)
-    )
+    return path_to_return
 
 
-def get_image_metadata_from_file_ext(file_ext):
+def get_image_metadata_from_file(file_like):
     """
-    Receives a valid image file format and returns a 2-tuple of two strings:
+    Receive a valid image file and returns a 2-tuple of two strings:
         [0]: Image format (i.e. 'jpg', 'gif' or 'png')
         [1]: InMemoryUploadedFile-friendly save format (i.e. 'image/jpeg')
     image_format, in_memory_file_type
     """
-    if file_ext not in FILE_EXTENSION_MAP:
-        return JPEG
-    else:
-        return FILE_EXTENSION_MAP[file_ext]
+    mime_type = magic.from_buffer(file_like.read(1024), mime=True)
+    file_like.seek(0)
+    image_format = MIME_TYPE_TO_PIL_IDENTIFIER[mime_type]
+    return image_format, mime_type
 
 
 def validate_versatileimagefield_sizekey_list(sizes):
     """
+    Validate a list of size keys.
+
     `sizes`: An iterable of 2-tuples, both strings. Example:
     [
         ('large', 'url'),
@@ -184,10 +165,6 @@ def validate_versatileimagefield_sizekey_list(sizes):
     ]
     """
     try:
-        sizes = list(sizes)
-    except TypeError:
-        raise InvalidSizeList('`sizes` must be an iterable')
-    else:
         for key, size_key in sizes:
             size_key_split = size_key.split('__')
             if size_key_split[-1] != 'url' and (
@@ -199,11 +176,16 @@ def validate_versatileimagefield_sizekey_list(sizes):
                     "by double underscores. Examples: 'crop__400x400', "
                     "filters__invert__url".format(size_key)
                 )
+    except ValueError:
+        raise InvalidSizeKeySet(
+            '{} is an invalid size key set. Size key sets must be an '
+            'iterable of 2-tuples'.format(str(sizes))
+        )
     return list(set(sizes))
 
 
 def get_url_from_image_key(image_instance, image_key):
-    """"""
+    """Build a URL from `image_key`."""
     img_key_split = image_key.split('__')
     if 'x' in img_key_split[-1]:
         size_key = img_key_split.pop(-1)
@@ -212,41 +194,42 @@ def get_url_from_image_key(image_instance, image_key):
     img_url = reduce(getattr, img_key_split, image_instance)
     if size_key:
         img_url = img_url[size_key].url
-    else:
-        img_url = image_instance.url
     return img_url
 
 
-def build_versatileimagefield_url_set(image_instance, size_set):
+def build_versatileimagefield_url_set(image_instance, size_set, request=None):
     """
-    Returns a dictionary of urls corresponding to size_set
+    Return a dictionary of urls corresponding to size_set
     - `image_instance`: A VersatileImageFieldFile
     - `size_set`: An iterable of 2-tuples, both strings. Example:
-    [
-        ('large', 'url'),
-        ('medium', 'crop__400x400'),
-        ('small', 'thumbnail__100x100')
-    ]
+        [
+            ('large', 'url'),
+            ('medium', 'crop__400x400'),
+            ('small', 'thumbnail__100x100')
+        ]
 
-    The above would lead to the following response:
-    {
-        'large': 'http://some.url/image.jpg',
-        'medium': 'http://some.url/__sized__/image-crop-400x400.jpg',
-        'small': 'http://some.url/__sized__/image-thumbnail-100x100.jpg',
-    }
+        The above would lead to the following response:
+        {
+            'large': 'http://some.url/image.jpg',
+            'medium': 'http://some.url/__sized__/image-crop-400x400.jpg',
+            'small': 'http://some.url/__sized__/image-thumbnail-100x100.jpg',
+        }
+    - `request`:
     """
     size_set = validate_versatileimagefield_sizekey_list(size_set)
     to_return = {}
-    if image_instance:
+    if image_instance or image_instance.field.placeholder_image:
         for key, image_key in size_set:
             img_url = get_url_from_image_key(image_instance, image_key)
+            if request is not None:
+                img_url = request.build_absolute_uri(img_url)
             to_return[key] = img_url
     return to_return
 
 
-def get_rendition_key_set(key, validate=True):
+def get_rendition_key_set(key):
     """
-    Retrieves a validated and prepped Rendition Key Set from
+    Retrieve a validated and prepped Rendition Key Set from
     settings.VERSATILEIMAGEFIELD_RENDITION_KEY_SETS
     """
     try:
@@ -257,7 +240,4 @@ def get_rendition_key_set(key, validate=True):
             "settings.VERSATILEIMAGEFIELD_RENDITION_KEY_SETS['{}']".format(key)
         )
     else:
-        if validate is True:
-            return validate_versatileimagefield_sizekey_list(rendition_key_set)
-        else:
-            return rendition_key_set
+        return validate_versatileimagefield_sizekey_list(rendition_key_set)
